#!/usr/bin/env python
# -*- coding: latin-1; -*-

# Copyright 2020 University of Liège
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from builtins import range
import numpy as np
import os
import shutil

import Metafor.Mparams.TaskManager as tm
import Metafor.Mparams.mparams as mp


def main():
    if tm.rank == 0:
        #--------------------------------------------------------
        #                       Model parameters
        #--------------------------------------------------------
        model_parameters = {}
        isUnix = lambda: os.name == 'posix'

        if isUnix():
            model_parameters['Metafor_exe']  = '/Users/kliegeois/dev/Metafor/oo_metaB/bin/Metafor'
            model_parameters['Metafor_model'] = '/Users/kliegeois/dev/waves/Metafor/models/bord01'
        else:
            model_parameters['Metafor_exe']  = 'C:/Users/kimli_000/dev/oo_metaB/bin/Release/Metafor'
            model_parameters['Metafor_model'] = 'C:/Users/kimli_000/dev/waves/Metafor/models/bord01'
        #--------------------------------------------------------
        #                    Surrogate parameters
        #--------------------------------------------------------
        surrogate_parameters = {}
        surrogate_parameters['display'] = False
        if not isUnix():
            surrogate_parameters['display'] = False
        surrogate_parameters['normalisation'] = False
        #--------------------------------------------------------
        surrogate_parameters['fname'] = 'mparams.log'
        d = 6
        surrogate_parameters['d'] = d
        surrogate_parameters['P_name'] = np.array([ 'NRTol', 'peno', 'peta', 'PEAS', 'nx', 'ny'])

        M = np.zeros((d,), dtype =np.int)
        M[0] = 1
        M[1] = 1
        M[2] = 1
        M[3] = 1
        M[4] = 1
        M[5] = 4

        surrogate_parameters['M'] = M

        surrogate_parameters['Dom_min'] = np.array([ 1400., 350.])
        surrogate_parameters['Dom_max'] = np.array([ 1600., 450.])

        N = np.zeros((d,), dtype =np.int)
        for j in range(0,d):
            N[j] = 2

        surrogate_parameters['N'] = N
        #--------------------------------------------------------
        #--------------------------------------------------------
        surrogate_parameters['function_type'] = 'monomial_1norm'
        surrogate_parameters['training_points'] = 'precomputed'
        surrogate_parameters['precomputed_training_points'] = '/Users/kliegeois/dev/waves/Metafor/models/bord01/numericalSATrainingPoints3.ascii'
        surrogate_parameters['precomputed_weights'] = '/Users/kliegeois/dev/waves/Metafor/models/bord01/numericalSAWeights3.ascii'
        surrogate_parameters['solver'] = 'QR'
        surrogate_parameters['means'] = np.array([1495.,396.])
        surrogate_parameters['variances'] = np.array([1390.,660.])
        rho = -0.233
        surrogate_parameters['correlations'] = np.array([[1.,rho],[rho,1.]])
        surrogate_parameters['f_mapping_method'] = 'Cholesky'
        surrogate_parameters['rho'] = rho
        surrogate_parameters['nstddeviation'] = 2.6281407
        #--------------------------------------------------------
        surrogate_parameters['m1'] = 40
        surrogate_parameters['m2'] = 40
        #--------------------------------------------------------
        #                    Images parameters
        #--------------------------------------------------------
        images_parameters = {}

        if tm.rank == 0:
            ##

            sol, Z, W_sr, s_hat = mp.master(model_parameters,surrogate_parameters,images_parameters)

            ##

            import codecs, json

            tmp = sol.tolist()
            file_path = "sol.json"
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4)

            tmp = Z.tolist()
            file_path = "Z.json"
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4)

            tmp = W_sr.tolist()
            file_path = "W_sr.json"
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4)

            tmp = s_hat.tolist()
            file_path = "s_hat.json"
            json.dump(tmp, codecs.open(file_path, 'w', encoding='utf-8'), separators=(',', ':'), sort_keys=True, indent=4)

        else:
            mp.worker()

if __name__ == "__main__":
    main()
