/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WFUN0POS_H
#define WFUN0POS_H

#include "flow.h"
#include "wObject.h"
#include <Eigen/Dense>

namespace flow
{

/**
 * @brief Scalar functions depending on spatial position
 * @authors Adrien Crovato
 */
class FLOW_API F0Ps : public fwk::wObject
{
public:
#ifndef SWIG
    F0Ps() : wObject()
    {
    }
#endif
    virtual ~F0Ps()
    {
    }
#ifndef SWIG
    virtual double eval(Eigen::Vector3d const &pos) const = 0;
#endif
};

/**
 * @brief Constant function
 * @authors Adrien Crovato
 */
class FLOW_API F0PsC : public F0Ps
{
    double v;

public:
    F0PsC(double _v) : F0Ps(), v(_v) {}

#ifndef SWIG
    virtual double eval(Eigen::Vector3d const &pos) const override;
    virtual void write(std::ostream &out) const override;
#endif
};

/**
 * @brief Freestream potential
 * @authors Adrien Crovato
 */
class FLOW_API F0PsPhiInf : public F0Ps
{
    int nDim;     ///< number of dimensions
    double alpha; ///< angle of attack

public:
    F0PsPhiInf(int _nDim, double _alpha);
    void update(double _alpha);

#ifndef SWIG
    virtual double eval(Eigen::Vector3d const &pos) const override;
    virtual void write(std::ostream &out) const override;
#endif
};

} // namespace flow

#endif //WFUN0POS_H
