#!/usr/bin/env python
# -*- coding: utf-8 -*-

from builtins import str
from builtins import range
from katropton.eigenvalues import *
from katropton.readers import *
from katropton.readers import *
import os
import fwk
"""@package docstring
This file is used to investigate the choice of prec parameters.
"""


def read_timer_all(n_cases, sub_cases, file_MC_base_dir):
    querylines = ['Belos: Operation Op*x',
                  'Belos: Operation Prec*x',
                  'Belos: DGKS[2]: Orthogonalization',
                  'Belos: DGKS[2]: Ortho (Inner Product)',
                  'Belos: DGKS[2]: Ortho (Norm)',
                  'Belos: DGKS[2]: Ortho (Update)',
                  'Belos: PseudoBlockGmresSolMgr total solve time']
    timer = np.zeros((n_cases, len(sub_cases), len(querylines)))
    for i in range(0, n_cases):
        file_MC_base_dir_2 = file_MC_base_dir + '/' + str(i)+'/'
        for j in range(0, len(sub_cases)):
            input_MC_file_dir = file_MC_base_dir_2 + str(sub_cases[j])+'/'
            timer[i, j, :] = read_teuchos_timers(
                input_MC_file_dir+'teuchos_timers.txt', querylines)
    return timer


def read_iter_all(n_cases, sub_cases, file_MC_base_dir):
    iter = np.zeros((n_cases, len(sub_cases)))
    for i in range(0, n_cases):
        file_MC_base_dir_2 = file_MC_base_dir + '/' + str(i)+'/'
        for j in range(0, len(sub_cases)):
            input_MC_file_dir = file_MC_base_dir_2 + str(sub_cases[j])+'/'
            iter_indices, residuals = read_Belos(
                input_MC_file_dir + 'belos_out.txt', 1)
            iter[i][j] = iter_indices[-1]
    return iter


def main():
    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    file_MC_base_dir = '/Volumes/HD_SONY/CMAME/Plate/gaussian/katropton_CMAME_tests_plate_correlation/'
    #file_dir + '/../../../workspace/katropton_CMAME_tests_plate_mean_prec_GS_direct/'
    sub_cases = ['1', '8']

    N = 200
    correlation_min_exp = -1
    correlation_max_exp = np.log10(2)

    correlations = np.logspace(
        correlation_min_exp, correlation_max_exp, N, endpoint=True)

    iter = read_iter_all(N, sub_cases, file_MC_base_dir)
    timer = read_timer_all(N, sub_cases, file_MC_base_dir)

    from mpl_toolkits.mplot3d import Axes3D
    import matplotlib.pyplot as plt
    from matplotlib import cm

    plt.subplot(211)
    plt.semilogx(correlations, iter[:, 0])
    plt.semilogx(correlations, iter[:, 1])
    plt.xlabel('correlation length')
    plt.ylabel('iter')
    plt.subplot(212)
    plt.semilogx(correlations, np.divide(iter[:, 1], iter[:, 0]))
    plt.xlabel('correlation length')
    plt.ylabel('R')

    plt.figure()
    plt.subplot(211)
    plt.plot(1./correlations, iter[:, 0])
    plt.plot(1./correlations, iter[:, 1])
    plt.xlabel('1./correlation length')
    plt.ylabel('iter')
    plt.subplot(212)
    plt.plot(1./correlations, np.divide(iter[:, 1], iter[:, 0]))
    plt.xlabel('1./correlation length')
    plt.ylabel('R')
    plt.show()


if __name__ == "__main__":
    main()
