#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div
import katropton as m
import tbox
import tbox.gmsh as gmsh
#import matplotlib.pyplot as plt

from katropton.eigenvalues import *
from katropton.convergence import *

import shlex
import subprocess
import os
import numpy as np

from PyTrilinos import Teuchos


def evaluate_cubes(args, msh, comm, ensemble_size, _E, file_dir):

    pbl = m.Problem(msh, comm)

    #E = np.linspace(0.5,1.5,ensemble_size)
    E = 10. * np.ones(ensemble_size)
    nu = 0.4 * np.ones(ensemble_size)
    k = 0.114
    d = 0.
    m.Medium(pbl, "Body 1", "test", E, nu, k, d)
    m.Medium(pbl, "Body 2", "test", E, nu, k, d)
    m.Dirichlet(pbl, "Clamped", "Clamped", 1, 0.,
                 1, 0., 1, 0., 0, 0., ensemble_size)
    #m.Dirichlet(pbl, "Clamped xy", "Clamped",1,0.,1,0.,1,0.,0,0.,ensemble_size)

    norm = tbox.Vector3d(0, 0, -1)
    cont = m.Contact(pbl, "Contact 2", "contact", norm)
    cont.setMeshTying()
    cont.setMaster(pbl, "Contact 1", norm)

    zero = np.zeros(ensemble_size)
    p = -0.2 * np.ones(ensemble_size)

    m.Neumann(pbl, "Load", "Load 1", 0, zero, 0,
              zero, 1, p, 0, zero, ensemble_size)

    solverList = Teuchos.ParameterList()
    solverList['Convergence Tolerance'] = 10**(-8)
    solverList['Maximum Iterations'] = 1000
    solverList['Verbosity'] = 33
    solverList['Flexible Gmres'] = False
    solverList['Output Style'] = 1
    solverList['Output Frequency'] = 1

    solverList['gamma'] = 29.58164477018121

    solverList["MueLu xml file name"] = file_dir + \
        '/../preconditioners/SIMPLE_gs.xml'
    solverList["convert MueLu xml file"] = True

    solverList['Use preconditioner'] = True

    solverList['type'] = "BlockGmres"
    solverList['Maximum active set iteration'] = 1

    solverList['Print Teuchos timers'] = True
    solverList['Write matrix and vectors'] = True
    solverList['Write txt files'] = True
    if ensemble_size == 1:
        solverList['Create Preconditioned Matrix'] = True

    if ensemble_size > 1:
        slv = m.IterativeSolver(pbl, args.k, solverList,
                                3, True, ensemble_size)
    else:
        slv = m.IterativeSolver(pbl, args.k, solverList,
                                3, ensemble_size)

    slv.start()


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    from fwk.wutils import parseargs
    args = parseargs()

    geo_name = '../tests/2_cubes_patch.geo'
    mesh_name = '2_cubes_patch.msh'
    partitioned_mesh_name = '2_cubes_patch.msh'

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    gmsh.MeshLoader(geo_name, __file__).execute()

    if rank == 0:
        command_line = 'gmsh -3 ' + work_dir + '/' + mesh_name + ' -o ' + \
            work_dir + '/' + partitioned_mesh_name + ' -part ' + str(siz)
        print(command_line)
        tmp = shlex.split(command_line)
        fileout = open('gmsh.log', 'w')
        p = subprocess.Popen(tmp, stdin=subprocess.PIPE, stdout=fileout,
                             stderr=fileout, env=os.environ, shell=False, close_fds=True)
        retcode = p.wait()
        for i in range(1, siz):
            comm.send(1, dest=i, tag=11)
    else:
        data = comm.recv(source=0, tag=11)

    if siz == 1:
        msh = gmsh.MeshLoader(mesh_name, work_dir).execute()
    else:
        msh = gmsh.MeshLoader(mesh_name, work_dir).myrank_execute(rank)

    N = 32

    E = np.linspace(5., 15.0, num=N)

    ensemble_sizes = [1, 8, 16, 32]

    for e in range(0, len(ensemble_sizes)):
        directory = str(ensemble_sizes[e])
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)

        for i in range(0, old_div(N, ensemble_sizes[e])):
            directory = str(i)
            if not os.path.exists(directory):
                os.makedirs(directory)
            os.chdir(directory)
            evaluate_cubes(
                args, msh, comm, ensemble_sizes[e], E[i*ensemble_sizes[e]:(i+1)*ensemble_sizes[e]], file_dir)
            os.chdir('..')

        os.chdir('..')


if __name__ == "__main__":
    main()
