#!/usr/bin/env python
# -*- coding: latin-1; -*-

from __future__ import print_function
from builtins import str
from builtins import range
import katropton.CMAME.model as model

import katropton as m
import tbox
import tbox.gmsh as gmsh
import shlex
import subprocess
import os
import numpy as np

from fwk.wutils import parseargs

from PyTrilinos import Teuchos

import katropton.utilities as u


def evaluate_correlation(correlation, msh, comm, file_dir, ensemble_size=1):

    work_dir = os.getcwd()
    args = parseargs()

    p = model.defaut_params

    N = ensemble_size
    num_random_variables = 20
    randomVariable = np.zeros((N, num_random_variables))

    nuw = 128
    p['Correlation length'] = correlation
    p['Dispersion level'] = 0.1

    p['Wavenumber cutoff'] = 8.
    p['Wavenumber discretization points'] = nuw
    p['Number Z of previously drawn samples'] = 0
    p['Number Phi of previously drawn samples'] = 0
    p['Is exp'] = False

    pm = model.plate_model(ensemble_size, randomVariable, msh, comm, file_dir,
                           work_dir, p)
    pm.run(args.k)


def evaluate_all_correlation(correlations, msh, comm, file_dir):

    N = correlations.shape[0]
    for i in range(0, N):
        directory = str(i)
        if not os.path.exists(directory):
            os.makedirs(directory)
        os.chdir(directory)
        for ensemble_size in [1, 8]:
            directory = str(ensemble_size)
            if not os.path.exists(directory):
                os.makedirs(directory)
            os.chdir(directory)
            evaluate_correlation(
                correlations[i], msh, comm, file_dir, ensemble_size)
            os.chdir('..')
        os.chdir('..')


def main():
    try:
        import mpi4py.MPI as mpi
        comm = mpi.COMM_WORLD
        rank = comm.rank
        siz = comm.size
        name = mpi.Get_processor_name()
        status = mpi.Status()
        print("info: MPI found")
    except:
        comm = None
        rank = 0
        siz = 1
        name = "noname"
        print("info: MPI not found => MPI disabled")

    args = parseargs()

    file_dir = os.path.dirname(__file__)
    work_dir = os.getcwd()

    # Units in the meshes are in mm
    msh = u.fuse_meshes('plate_1.msh', 'plate_1_tmp', 'plate_2.msh',
                        'plate_2_tmp', 'plate_fused', file_dir, work_dir)

    N = 200
    correlation_min_exp = -1
    correlation_max_exp = np.log10(2)

    correlations = np.logspace(
        correlation_min_exp, correlation_max_exp, N, endpoint=True)

    np.savetxt('correlations.txt', correlations)

    evaluate_all_correlation(correlations, msh, comm, file_dir)


if __name__ == "__main__":
    main()
