#ifndef KATROPTON_ELEMENTMATRICES_H
#define KATROPTON_ELEMENTMATRICES_H

#include "Stokhos_Sacado_Kokkos_MP_Vector.hpp"
#include "EnsembleTraits.h"

/*
#include "Stokhos_Tpetra_MP_Vector.hpp"
#include "Stokhos_Tpetra_Utilities_MP_Vector.hpp"
#include "Stokhos_Sacado_Kokkos_MP_Vector.hpp"
*/

#include <Kokkos_Core.hpp>
#include "Kokkos_ViewFactory.hpp"
#include "wCacheHex8.h"
#include "wCacheQuad4.h"
#include "wCacheTetra4.h"
#include "wCacheTri3.h"
#include "tMatrix.h"

//#include "Sacado_Fad_MP_Vector.hpp"
//#include "Sacado.hpp"
#include "Domain.h"

#include "MaterialsList.h"
#include "Material.h"

#include "wLazy.h"
#include "tMatrix.h"

#include "ElementComputation.h"

#include "EnsembleTraits.h"

#include <iostream>

/**
 * @brief Class used to compute the element matrices:
 * <ul>
 *  <li> The element stifness matrix \f$\boldsymbol{K}\f$,
 *  <li> The thermal element matrix \f$\boldsymbol{L}\f$,
 *  <li> The coupling element matrix \f$\boldsymbol{S}\f$.
 * </ul> 
 */
template <typename scalar, int element_type, int element_size>
class ElementMatrices : public ElementComputation<scalar, element_type>
{
public:
    typedef Kokkos::View<scalar ***, Kokkos::LayoutRight, Kokkos::DefaultExecutionSpace> view_type_3D;
    typedef Kokkos::View<scalar **, Kokkos::LayoutRight, Kokkos::DefaultExecutionSpace> view_type_2D;

    view_type_3D K, L, S, K_e, L_e, S_e;

    view_type_2D material;

    Teuchos::RCP<katropton::Domain<scalar>> domain;

    size_t numPrimalDPN;

    ElementMatrices(Teuchos::RCP<katropton::Domain<scalar>> domain, size_t numPrimalDPN, size_t pool_size);
    ~ElementMatrices(){};

    void compute(int e, int i_thread);
    view_type_2D getK(size_t i_thread) const;
    view_type_2D getL(size_t i_thread) const;
    view_type_2D getS(size_t i_thread) const;
};

#endif //KATROPTON_ELEMENTMATRICES_H
