#ifndef KATROPTON_NEUMANN_H
#define KATROPTON_NEUMANN_H

#include "katropton.h"
#include "wGroup.h"
#include <vector>
#include <string>

namespace katropton
{

/**
 * @brief Class used to specify in Python the different Neumann boundary conditions 
 * (both mechanical and thermal).
 */
class KATROPTON_API Neumann : public tbox::Group
{
public:
    std::string const &BC_name;
    std::vector<tbox::Node *> nodes;
    std::vector<int> which_dof;
    std::vector<double> values;

    Neumann(Problem &pbl,
            std::string const &name,
            std::string const &_BC_name,
            std::vector<int> _which_dof,
            std::vector<double> _values);

    Neumann(Problem &pbl,
            std::string const &name,
            std::string const &_BC_name,
            int x,
            std::vector<double> v_x,
            int y,
            std::vector<double> v_y,
            int z,
            std::vector<double> v_z,
            int T,
            std::vector<double> v_T,
            size_t ensemble_size);

#ifndef SWIG
    virtual void write(std::ostream &out) const;
#endif
};

} // namespace katropton

#endif //KATROPTON_NEUMANN_H
