#ifndef KATROPTON_WEIGHT_H
#define KATROPTON_WEIGHT_H

#include "katropton.h"
#include "wGroup.h"
#include <vector>
#include <string>

#include <Teuchos_RCP.hpp>
using namespace tbox;

namespace katropton
{

/**
 * @brief Class used to specify in Python the different weights used for
 * the computation of the weighted residual of the GMRES.
 */
class KATROPTON_API Weight : public Group
{
public:
    std::vector<Node *> nodes;
    std::vector<int> which_dof;
    std::vector<double> values;

    std::vector<double> x_values;
    std::vector<double> y_values;
    std::vector<double> z_values;
    std::vector<double> T_values;

    Weight(Problem &pbl,
           std::string const &name,
           std::vector<int> _which_dof,
           std::vector<double> _values,
           size_t ensemble_size = 1);

    Weight(Problem &pbl,
           std::string const &name,
           int x,
           double v_x,
           int y,
           double v_y,
           int z,
           double v_z,
           int T = 0,
           double v_T = 0,
           size_t ensemble_size = 1);

    Weight(Problem &pbl,
           std::string const &name,
           int x,
           std::vector<double> v_x,
           int y,
           std::vector<double> v_y,
           int z,
           std::vector<double> v_z,
           int T = 0,
           std::vector<double> v_T = {});

#ifndef SWIG
    virtual void write(std::ostream &out) const;
#endif
};

} // namespace katropton

#endif //KATROPTON_WEIGHT_H
