#!/bin/bash
# Submission script for Blake
#SBATCH --job-name=test_4
#SBATCH --time=41:00:00 # hh:mm:ss
#
#SBATCH -N 4
#SBATCH -p blake
#
#SBATCH --comment=test
#SBATCH --output=workspace/test_4.%j.out

. ~/.bash_profile $SLURM_SUBMIT_HOST

export MKL_DYNAMIC=TRUE
export OMP_DYNAMIC=FALSE
export OMP_NUM_THREADS=24
export KMP_HW_SUBSET=24c,1t
export KMP_AFFINITY=compact

export MKL_NUM_THREADS=${OMP_NUM_THREADS}

n_MPI=8

i_min=1
i_max=201

m_1=10
m_2=100
m_3=200

nCx=100
nCy=100

base_dir=$(pwd)
geo_file=${base_dir}/katropton/thesis/test_4/1_cube.geo
workspace_dir_0=${base_dir}/workspace/katropton_thesis_test_4_1_cube
workspace_dir=${workspace_dir_0}/MPI_${n_MPI}

if [ ! -d ${workspace_dir_0} ] 
then
    mkdir ${workspace_dir_0}
    mkdir ${workspace_dir}
fi

if [ ! -d ${workspace_dir} ] 
then    
    mkdir ${workspace_dir}
fi

for (( i = $i_min; i < $i_max; ++i )); do
  nCz=$(( 5*i ))
  msh_file=${workspace_dir}/1_cube_${i}.msh
  gmsh -3 ${geo_file} -setnumber nCx ${nCx} -setnumber nCy ${nCy} -setnumber nCz ${nCz} -o ${msh_file} -part ${n_MPI} -format msh22
  echo $i &> ${workspace_dir}/i.txt
  echo $m_1 &> ${workspace_dir}/m.txt
  mpirun -npersocket 1 python run.py -k ${OMP_NUM_THREADS} katropton/thesis/test_4/1_cube.py
  echo $m_2 &> ${workspace_dir}/m.txt
  mpirun -npersocket 1 python run.py -k ${OMP_NUM_THREADS} katropton/thesis/test_4/1_cube.py
  echo $m_3 &> ${workspace_dir}/m.txt
  mpirun -npersocket 1 python run.py -k ${OMP_NUM_THREADS} katropton/thesis/test_4/1_cube.py
done
