#!/usr/bin/env python
# -*- coding: utf-8 -*-
# simplified model of a mirror and a holder 
# (two rectangular blocks on top of each other)

import mirrors as m
import tbox
import tbox.gmsh as gmsh

def model(p={}):
    # configure/build mesh

    # [RB] what is the purpose of this dictionnary 
    # why couldn't we use "p" instead??
    pars = { 'L' : p['L'] , 
    'L_1' : p['L_1'], 
    'H' : p['H'], 
    'H_1' : p['H_1'], 
    'W' : p['W'], 
    'nL' : p['nL'], 
    'nL_1' : p['nL_1'], 
    'nH' : p['nH'], 
    'nH_1' : p['nH_1'], 
    'nW' : p['nW'] }

    msh = gmsh.MeshLoader("mirror.geo", __file__).execute(**pars)
    pbl = m.Problem(msh, "thermomechanical", 22)

    # configure pbl
    m.Medium(pbl, "Holder", "TZM", 330000, 0.36, 0.114, 0.5E-05)
    m.Medium(pbl, "Mirror", "ScMo", 330000, 0.36, 0.114, 0.4E-05)

    clamped = tbox.Vector3d(1, 1, 1)
    values_clamped = tbox.Vector3d(0, 0, 0)

    xyclamped = tbox.Vector3d(1, 1, 0)
    xzclamped = tbox.Vector3d(1, 0, 1)

    T = p['T']
    m.uDirichlet(pbl, "Vertical clamped line", 
                      "Vertical clamped line_name", 
                      xyclamped, values_clamped)
    m.uDirichlet(pbl, "Horizontal clamped line", 
                      "Horizontal clamped line_name", 
                      xzclamped, values_clamped)
    m.TDirichlet(pbl, "Clamped Side", "Clamped", T)

    return (pbl, msh) # retourner pbl sinon il semble detruit...


def runmodel(**d):
    
    p={}
    p['L']    = 250.0
    p['L_1']  = 250.0
    p['H']    = 50.0
    p['H_1']  = 30.0
    p['W']    = 150.0
    p['nL']   = 21
    p['nL_1'] = 21
    p['nH']   = 6
    p['nH_1'] = 3
    p['nW']   = 25
    p['T']    = 70.
    p['nogui'] = False
    p.update(d)
    
    # builds an instance of the model with given parameters
    (pbl, msh) = model(p)
    gmshWriter = tbox.GmshExport(msh)

    # retrieves command-line arguments
    from fwk.wutils import parseargs
    args = parseargs()

    # creates a solver and runs it
    from tbox.solvers import LinearSolver
    solver = m.Solver(pbl, LinearSolver().mumps(), args.k)
    solver.start(gmshWriter)

    if not args.nogui and not p['nogui']:
        from mirrors.viewer import GUI
        GUI().open('mirror')

    return pbl.Sol

if __name__ == "__main__":
    runmodel()
