/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WGMSHIMPORT_H
#define WGMSHIMPORT_H

#include "tbox.h"
#include "wMshImport.h"
#include <vector>
#include <map>

namespace tbox
{

/**
 * @brief Read and load gmsh mesh
 * @authors Adrien Crovato, Romain Boman
 */
class TBOX_API GmshImport : public MshImport
{
    std::map<int, Node *> nodemap; ///< Map between node and its id number

    int searchCode(char const *code, char const *line) const;
    void readMshFormat(FILE *file);
    void readMshNodes(FILE *file);
    void readMshElements(FILE *file, int myrank);
    void readMshPhyNames(FILE *file);
    void readElNods(FILE *file, int n, std::vector<Node *> &nods);
    Tag *addTag(FILE *file, std::map<int, Tag *> &tmap);

public:
    GmshImport(std::shared_ptr<MshData> _msh);
    virtual ~GmshImport();

    virtual void load(std::string const &fname, int myrank) override;

    virtual void write(std::ostream &out) const override;
};

} // namespace tbox

#endif //WGMSHIMPORT_H
