/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLAZY_H
#define WLAZY_H

#include "tbox.h"

/* @todo usefull? clean? */
#if WAVES_USE_TBB
#include <tbb/mutex.h>
#include <tbb/atomic.h>
#endif

namespace tbox
{

/**
 * @brief TBB implementation of a "lazy initialization"
 *        (thread safe allocation of a shared pointer)
 * @todo clean
 */

template <typename T>
class Lazy
{
#if WAVES_USE_TBB
    tbb::atomic<T *> value;
    tbb::mutex mut;
#else
    T *value;
#endif
    int np;

public:
    Lazy(int _np) : value(), np(_np) {}
    ~Lazy()
    {
        delete value;
        //std::cout << "lazy pointer deleted\n";
    }
    T &get()
    {
        if (!value)
        {
#if WAVES_USE_TBB
            tbb::mutex::scoped_lock lock(mut);
#endif
            if (!value)
                value = new T(np);
        }
        return *value;
    }
};

} // namespace tbox

#endif //WLAZY_H
