/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WLINEARSOLVER_H
#define WLINEARSOLVER_H

#include "tbox.h"
#include "wObject.h"
#include <Eigen/Sparse>

namespace tbox
{

/**
 * @brief Base interface class for linear solvers
 * @authors Adrien Crovato
 */
class TBOX_API LinearSolver : public fwk::wSharedObject
{
public:
    LinearSolver() {}
    virtual ~LinearSolver() {}

#ifndef SWIG
    virtual void analyze(Eigen::SparseMatrix<double> const &A);
    virtual void factorize(Eigen::SparseMatrix<double> const &A);
    virtual void solve(Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x);
    virtual void compute(Eigen::SparseMatrix<double> const &A, Eigen::Map<Eigen::VectorXd> const &b, Eigen::Map<Eigen::VectorXd> &x);

    virtual double getError();
    virtual int getIterations();
#endif
};

} // namespace tbox

#endif // WLINEARSOLVER_H