/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wMem.h"
#include "wElement.h"

using namespace tbox;

Mem::Mem(size_t _ngp) : ngp(_ngp), detJs(_ngp), Js(_ngp), Jinvs(_ngp)
{
    valid = false;
    vol = 0;
    cg = Eigen::Vector3d::Zero();
}

/**
 * @brief Check that determinant of Jacobian is positive
 */
void Mem::checkJac(Element const *e, double detJ) const
{
    if (detJ <= 0)
    {
        std::stringstream err;
        err << "Negative jacobian detected for element: " << *e << "\n";
        throw std::runtime_error(err.str());
    }
}

void Mem::update(bool sameDim)
{
    throw std::runtime_error("Mem::update() not implemented!\n");
}
