/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "wMemQuad4.h"
#include "wNode.h"

using namespace tbox;

MemQuad4::MemQuad4(Quad4 &_e, size_t _ngp) : Mem(_ngp), e(_e)
{
}

/**
 * @brief Update Quad4 values
 */
void MemQuad4::update(bool sameDim)
{
    // Quad4 is not of same dimension as Problem (surface element)
    if (!sameDim)
    {
        for (int k = 0; k < ngp; ++k)
            detJs[k] = e.buildJ(k);
    }
    // Quad4 is of same dimension as Problem (volume element)
    else
    {
        for (int k = 0; k < ngp; ++k)
        {
            // detJ, J
            detJs[k] = e.buildJ(k, Js[k]);
            checkJac(&e, detJs[k]);
            // Jinv
            Jinvs[k] = Js[k].inverse();
        }
    }
    // CG
    cg = Eigen::Vector3d::Zero();
    for (auto n : e.nodes)
        cg += n->pos;
    cg /= e.nodes.size();
    // Memory is now valid and the volume can be computed
    valid = true;
    vol = e.Element::computeV();
}
