/*
 * Copyright 2020 University of Liège
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef WTRI3_H
#define WTRI3_H

#include "tbox.h"
#include "wLazy.h"
#include "wElement.h"
#include "wCacheTri3.h"

namespace tbox
{

/**
 * @brief Linear triangle finite element
 * @authors Romain Boman, Adrien Crovato
 */
class TBOX_API Tri3 : public Element
{
#ifndef SWIG
    friend class MemTri3;
    MemTri3 *pmem; ///< private memory of precalculated values
    inline MemTri3 &getMem();
    inline static CacheTri3 &getCache();
    virtual double buildJ(size_t k, Eigen::MatrixXd &J) const override;
    virtual double buildJ(size_t k) const override;

    virtual Eigen::MatrixXd buildK(std::vector<double> const &u, Fct0 const &fct) override;
    virtual Eigen::MatrixXd buildK(std::vector<double> const &u, Fct2 const &fct, bool fake) override;
    virtual Eigen::MatrixXd buildK_mechanical(Eigen::MatrixXd const &H) override;
    virtual Eigen::VectorXd builds(std::vector<double> const &u, Fct0 const &f) override;
    virtual Eigen::VectorXd builds(std::vector<double> const &u, Fct1 const &f) override;
    virtual Eigen::MatrixXd buildDK(std::vector<double> const &u, Fct0 const &fct) override;
    virtual Eigen::VectorXd buildDs(std::vector<double> const &u, Fct0 const &f) override;
    virtual Eigen::MatrixXd buildS() override;

    virtual Eigen::VectorXd computeGrad(std::vector<double> const &u, size_t k) override;
    virtual double computeGrad2(std::vector<double> const &u, size_t k) override;
    virtual Eigen::VectorXd computeqV(std::vector<double> const &u, Fct2 const &fct) override;
    virtual Eigen::VectorXd computeqint(std::vector<double> const &u, Fct2 const &fct) override;
    virtual double computeV(std::vector<double> const &u, Fct0 const &fct) override;
    virtual Eigen::MatrixXd computeV(std::vector<double> const &u, Fct2 const &fct) override;
    virtual double computeV() override;

#endif

public:
    Tri3(int n, Tag *_ptag, Tag *_etag, std::vector<Tag *> &_parts, std::vector<Node *> &nods);
    virtual ~Tri3();
    virtual ELTYPE type() const override { return ELTYPE::TRI3; }

#ifndef SWIG
    virtual Mem &getVMem() override;
    virtual Cache &getVCache() override;
    virtual Eigen::Vector3d normal() override;
#endif
};

#include "wTri3.inl"

} // namespace tbox

#endif //WTRI3_H
